import { logger } from "@libp2p/logger";
import { assert } from "./utils.js";
export class Mempool {
    constructor() {
        this.log = logger("canvas:gossiplog:mempool");
        this.values = new Map();
        this.missingParents = new Map();
        this.waitingChildren = new Map();
    }
    add(id, value, missingParents) {
        this.values.set(id, value);
        this.missingParents.set(id, missingParents);
        for (const parent of missingParents) {
            const children = this.waitingChildren.get(parent);
            if (children === undefined) {
                this.waitingChildren.set(parent, new Set([id]));
            }
            else {
                children.add(id);
            }
        }
    }
    *observe(id) {
        const children = this.waitingChildren.get(id);
        this.log("observing %s with %d children %o", id, children?.size ?? 0, children);
        if (children === undefined) {
            return [];
        }
        this.waitingChildren.delete(id);
        for (const child of children) {
            const missingParents = this.missingParents.get(child);
            assert(missingParents !== undefined, "expected missingParents !== undefined");
            missingParents.delete(id);
            if (missingParents.size === 0) {
                const value = this.values.get(child);
                assert(value !== undefined, "expected value !== undefined");
                this.values.delete(child);
                this.missingParents.delete(child);
                yield [child, value];
            }
        }
    }
}
